/* Matrix and vector arithmetic.
 * By Richard W.M. Jones <rich@annexia.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: matvec.h,v 1.1 2001/02/08 12:51:30 rich Exp $
 */

#ifndef MATVEC_H
#define MATVEC_H

#include <pool.h>
#include <math.h>

/* Notes:
 *
 * This library only handles 4x4 matrices and 4-vectors, for
 * basic 3D graphics use. A matrix is just an array float[16]
 * and a vector is just an array float[4]. You can either allocate
 * these in a pool using ``new_matrix'' and ``new_vec'', or
 * else you can allocate them statically.
 *
 * All matrices are stored in COLUMN-MAJOR ORDER! This is for
 * compatibility with OpenGL, but it is the OPPOSITE of the
 * normal C row-major ordering, so beware. Matrix elements in
 * column-major order are as follows:
 *
 *   M = | m0  m4  m8  m12 |
 *       | m1  m5  m9  m13 |
 *       | m2  m6  m10 m14 |
 *       | m3  m7  m11 m15 |
 *
 * Some of these functions have been inlined. I only inline
 * functions based on evidence from profiling the code as a
 * whole or where the function is so simple that the overhead
 * of calling it is larger than the inlined code. Excessive
 * inlining can itself cause performance problems, particularly
 * on modern processors which are very good at making jumps
 * and function calls.
 *
 */

extern float identity_matrix[16];
extern float zero_vec[4];

#define new_matrix(pool) ((float *) pmalloc ((pool), sizeof (float) * 16))
#define new_vec(pool) ((float *) pmalloc ((pool), sizeof (float) * 4))

extern float *new_identity_matrix (pool);
extern float *new_zero_vec (pool);

#define make_identity_matrix(m) memcpy (m, identity_matrix, sizeof(float)*16);

#define make_zero_vec(v) memcpy (v, zero_vec, sizeof (float) * 4);

extern void make_rotation_matrix (float angle,
				  float x, float y, float z,
				  float *m);

extern void make_translation_matrix (float x, float y, float z, float *m);

extern void make_scaling_matrix (float x, float y, float z, float *m);

extern void matrix_euler_to_rotation (float angle_x, float angle_y,
				      float angle_z,
				      float *mat);
extern void matrix_rotation_to_euler (const float *mat,
				      float *angle_x, float *angle_y,
				      float *angle_z);

extern void matrix_multiply (const float *a, const float *b,
			     float *product);

extern void matrix_vec_multiply (const float *m, const float *v,
				 float *result);

extern float vec_magnitude (const float *v);
extern float vec_magnitude2d (const float *v);

extern void vec_normalize (const float *v, float *r);
extern void vec_normalize2d (const float *v, float *r);

extern void vec_unit_normal_to_side (float *side, float *normal);

extern float vec_dot_product (const float *v1, const float *v2);
extern float vec_magnitude_in_direction (const float *v1, const float *v2);

extern float vec_angle_between (const float *v1, const float *v2);
extern void vec_cross_product (const float *v, const float *w, float *r);

extern void vec_scale (const float *a, float n, float *r);
extern void vec_add (const float *a, const float *b, float *r);

extern void point_midpoint (const float *p1, const float *p2, float *mp);
extern void point_midpoint2d (const float *p1, const float *p2, float *mp);
extern float point_distance (const float *p, const float *q);

extern float point_distance_to_plane (const float *plane,
				      const float *point);
extern int point_is_inside_plane (const float *plane, const float *point);

extern float point_distance_to_line (const float *point,
				     const float *line_point,
				     const float *line_vector);

extern void plane_coefficients (const float *p, const float *q,
				const float *r, float *co);

extern void quaternion_normalize (const float *a, float *r);
extern void quaternion_conjugate (const float *a, float *r);
extern float quaternion_magnitude (const float *a);
extern void quaternion_multiply (const float *a, const float *b, float *r);
extern void quaternion_to_rotation_matrix (const float *q, float *mat);
extern void make_quaternion_from_axis_angle (const float *axis, float angle,
					     float *q);

#endif /* MATVEC_H */
